#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>

// *****************************************************************************
// * class LinuxProcessData:
// *	This is the raw data that is stored in the LinuxProcess class.
// *****************************************************************************
typedef struct 
	{
	char *   cmdline;	// The execNameand line used to start the process	
	
				// Items retrieved from the stat file
	pid_t    pid;		// Process ID
	char *   execName;		// Filename of the executable
	char     state;		// Process state
	pid_t    parent;	// Process ID of the parent
	
				// Items retrieved from /proc/pid ownership
	uid_t    uid;
	char *   user;
	gid_t    gid;
	char  *  group;
	} LinuxProcessData;

// *****************************************************************************
// * class LinuxProcess:
// *	This class represents one Linux process.
// *****************************************************************************
class LinuxProcess : public LinuxProcessData
{
protected:
	LinuxProcess * next;	// The next process in the list

public:
	LinuxProcess             ( int procID );
	~LinuxProcess            ( void );
	void           setNext   ( LinuxProcess * nextProc ) { next = nextProc; }
	LinuxProcess * getNext   ( void ) { return next; }
	void           asciiDump ( FILE * fp = stdout);
};	


// *****************************************************************************
// * class LinuxProcessReply :
// *	This is a collection of LinuxProcess objects that is returned when the
// *	caller makes a request for a collection of processes.
// *****************************************************************************
class LinuxProcessReply 
{
protected:
	size_t          count;
	size_t          max;
	LinuxProcess ** items;	
	
public:
	LinuxProcessReply          ( void );
	~LinuxProcessReply         ( void );
	void           addProcess  ( LinuxProcess * );
	LinuxProcess * operator [] ( size_t idx );
	void           asciiDump   ( FILE * fp = stdout);
};

// *****************************************************************************
// * class LinuxProcessList:
// *	This class represents a collection of LinuxProcess objects
// *****************************************************************************
class LinuxProcessList
{
protected:
	LinuxProcess * procs;
	LinuxProcess * tail;
	
public:
	LinuxProcessList                    ( void );
	~LinuxProcessList                   ( void );
	LinuxProcessReply * getProc         ( void );
	LinuxProcessReply * getProcByPid    ( pid_t pid );
	LinuxProcessReply * getProcByName   ( char * name );
	LinuxProcessReply * getProcByUser   ( uid_t uid );
	LinuxProcessReply * getProcByUser   ( char * name );
	LinuxProcessReply * getProcByGroup  ( gid_t gid );
	LinuxProcessReply * getProcByGroup  ( char * name );
	LinuxProcessReply * getProcByParent ( pid_t pid );
	void                asciiDump       ( FILE * fp = stdout);
};

