// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
// 	This header file contains the clases required tpo read through the
//	server log file and put them into a usable format.
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _SERVER_DATABASE_H_
#define _SERVER_DATABASE_H_ 1

#include "PBS_JobCollection.h"
#include "PBS_JobArray.h"
#include "PBS_JobRecord.h"
#include "PBS_Server.h"
#include <sys/param.h>

// *****************************************************************************
// * class PBS_ServerDatabase :
// *	This object contains a collection of PBS_JobRecords that are associated
// *	with a specific server and its' log files.  The PBS_JobRecords are 
// *	associated with PBS_JobArrays that identify the owner of the job and 
// *	PBS_JobArrays that identify where they are/were enqueued.
// *
// *	The PBS_ServerDatabase class defines the mechanisms that are used to 
// *	read and store the log files that are generated by a PBS Server.
// *****************************************************************************
class PBS_ServerDatabase : public PBS_JobCollection
{
protected:	
	off_t         dataFileOffset;
	char          dataFile[MAXPATHLEN];
	time_t        cutOffSeconds;
	
	char        * dataPath;
	PBSStringHash userHash;
	PBSStringHash queueHash;
	
	void readFile               ( FILE * fp );
	
public:	
	PBS_ServerDatabase          ( char * Name, char * DataPath = NULL, time_t days = 14 );
	~PBS_ServerDatabase         ( void );

	char *          getUserName ( const char * name );
	PBS_JobArray  * getQueue    ( const char * queue );
	PBS_JobArray  * getUser     ( const char * user  );	
	PBS_JobRecord * newJob      ( const char * job );

	void        initialize      ( void );
	void        update          ( void );
	void        update          ( PBS_Server & server );
	void        clear           ( void );
	void        purge           ( void );
	void        asciiDump       ( FILE * fp = stdout, char * prefix = "\0");
};

#endif /* _SERVER_DATABASE_H_ */
