// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description: The UnderLord header file contains the class definitions for
//               the UnderLord scheduler responsible for performing job
//               sorting, matching and scheduling on a single cluster.
// 
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _UNDERLORD_H_
#define _UNDERLORD_H_ 1

#include <PBS_Server.h>
#include <MatchMatrix.h>
#include <UnderLordStage.h>

class UnderLord
{
protected:
	PBS_Server         server;
	MatchMatrix        matrix;
	time_t             lastSummary;
	UnderLordStage   * stages;
	

	void   sortJobs        ( PBS_Server & server, PBS_Job ** jobs, size_t jobCnt, int record );
	void   deployJobs      ( PBS_Server & server, JobDeployment * scheduled );
	void   writeSummary    ( PBS_Server & server, JobDeployment * scheduled );

public:
	UnderLord              ( void );
	~UnderLord             ( void );
	void       addStage    ( UnderLordStage * stage );
	void       processJobs ( int connID );	
	static int compareFunc ( const void * a, const void * b );

	inline PBS_Server & getServer ( void ) { return server; }
};
	
#endif /* _UNDERLORD_H_ */
