#include <MatchMatrix.h>
#include <PBS_Server.h>
#include <unistd.h>
#include <JobDeployment.h>
#include <StopWatch.h>

// *****************************************************************************
// * This variable identifies this application to the logging methods.
// *****************************************************************************
const char * msg_daemonname = SetLogLocal("testMatrix");


StopWatch stopwatch;

int main (int argc, char ** argv)
{
int    connID = -1;
FILE * fp  = NULL;
char   buf[1024];

if(argc>1)
	{
	connID = pbs_connect(argv[1]);
	}
else if	((fp = fopen("/usr/spool/PBS/server_name", "r"))!=NULL &&
	(fgets(buf, 1024, fp)!=NULL))
	{
	// Remove the trailing carriage return
	char * ptr;
	if((ptr = strchr(buf, '\n'))!=NULL) *ptr = 0;

	connID = pbs_connect(buf);
	fclose(fp);
	}

if(connID<=0)
	{
	fprintf(stdout, "Error: Server name unspecified or not valid\n");
	return 0;
	}

MatchMatrix matrix;
PBS_Server  server( connID );
PBS_Job **  jobs = server.getJobsByState(PBS_Job::QUEUED);
size_t      jobCnt = 0;


for(jobCnt=0; jobs && jobs[jobCnt]; jobCnt++);

size_t qIter = 1000;
size_t cIter = 10;

JobDeployment * scheduled = NULL;

fprintf(stdout, "\n\tPERFORMING %i Iterations of Quick MatchMaking With %i Jobs\n", qIter, jobCnt);
stopwatch.start();
for(int idx=0; idx<qIter; idx++)
	{
	matrix.endCycle();
	matrix.startCycle(server);
	scheduled = matrix.processJobs(jobs, jobCnt, 1);
	}
stopwatch.stop();

while(scheduled!=NULL)
	{
	scheduled->asciiDump(stdout, "\t\t=> ");
	scheduled = scheduled->getNext();
	}
matrix.endCycle();
fprintf(stdout, "\t\t=>COMPLETED %i ITERATIONS IN %g SECONDS...\n", qIter, stopwatch.getReading());
fprintf(stdout, "\t\t\t=>%g ITERATIONS/SECOND\n", (double)qIter/stopwatch.getReading());
fprintf(stdout, "\t\t\t=>%g JOBS/SECOND\n", (double)(jobCnt*qIter)/stopwatch.getReading());

fprintf(stdout, "\n\tPERFORMING %i Iterations of Comprehensive MatchMaking with %i Jobs\n", cIter, jobCnt);
stopwatch.start();
for(int idx=0; idx<cIter; idx++)
	{
	matrix.endCycle();
	matrix.startCycle(server);
	scheduled = matrix.processJobs(jobs, jobCnt, 0);
	}
stopwatch.stop();

while(scheduled!=NULL)
	{
	scheduled->asciiDump(stdout, "\t\t=> ");
	scheduled = scheduled->getNext();
	}
matrix.endCycle();
	
	
fprintf(stdout, "\t\t=>COMPLETED %i ITERATIONS IN %g SECONDS...\n", cIter, stopwatch.getReading());
fprintf(stdout, "\t\t\t=>%g ITERATIONS/SECOND\n", (double)cIter/stopwatch.getReading());
fprintf(stdout, "\t\t\t=>%g JOBS/SECOND\n", (double)(jobCnt*cIter)/stopwatch.getReading());

return 0;
}
